@@ -206,7 +206,8 @@ gst_atomic_int_dec_and_test (GstAtomicInt *aint)
 }
 
 /***** Sun SPARC *****/
-#elif defined(HAVE_CPU_SPARC) && defined(__GNUC__)
+/* disabled for now */
+#elif defined(HAVE_CPU_SPARC) && defined(__GNUC__) && 0
 
 GST_INLINE_FUNC void 	gst_atomic_int_destroy 	(GstAtomicInt *aint) { } 
 
@@ -271,7 +272,7 @@ gst_atomic_int_dec_and_test (GstAtomicInt *aint)
   register int increment asm ("g2");
 
   ptr = &aint->counter;
-  increment = val;
+  increment = 1;
 
   __asm__ __volatile__(
     "mov    %%o7, %%g4\n\t"
@@ -337,6 +338,7 @@ gst_atomic_int_dec_and_test (GstAtomicInt *aint)
 
 /***** S/390 *****/
 #elif defined(HAVE_CPU_S390) && defined(__GNUC__)
+typedef struct { volatile int counter; } atomic_t __attribute__ ((aligned (4)));
 
 GST_INLINE_FUNC void 	gst_atomic_int_init 	(GstAtomicInt *aint, gint val) { aint->counter = val; }
 GST_INLINE_FUNC void 	gst_atomic_int_destroy 	(GstAtomicInt *aint) { } 
