@@ -49,13 +49,16 @@ static void 			gst_bin_dispose 		(GObject * object);
 static GstElementStateReturn	gst_bin_change_state		(GstElement *element);
 static GstElementStateReturn	gst_bin_change_state_norecurse	(GstBin *bin);
 
-static void 			gst_bin_set_index 		(GstBin *bin, GstIndex *index);
+static void 			gst_bin_set_index 		(GstElement *element, GstIndex *index);
 
-static gboolean 		gst_bin_iterate_func 		(GstBin * bin);
+static void 			gst_bin_add_func 		(GstBin *bin, GstElement *element);
+static void 			gst_bin_remove_func 		(GstBin *bin, GstElement *element);
+
+static gboolean 		gst_bin_iterate_func 		(GstBin *bin);
 
 #ifndef GST_DISABLE_LOADSAVE
-static xmlNodePtr 		gst_bin_save_thyself 		(GstObject * object, xmlNodePtr parent);
-static void 			gst_bin_restore_thyself 	(GstObject * object, xmlNodePtr self);
+static xmlNodePtr 		gst_bin_save_thyself 		(GstObject *object, xmlNodePtr parent);
+static void 			gst_bin_restore_thyself 	(GstObject *object, xmlNodePtr self);
 #endif
 
 /* Bin signals and args */
@@ -123,7 +126,6 @@ gst_bin_class_init (GstBinClass * klass)
 		  gst_marshal_VOID__POINTER, G_TYPE_NONE, 1, G_TYPE_POINTER);
 
   gobject_class->dispose 		= GST_DEBUG_FUNCPTR (gst_bin_dispose);
-  gobject_class->dispose 		= GST_DEBUG_FUNCPTR (gst_bin_dispose);
 
 #ifndef GST_DISABLE_LOADSAVE
   gstobject_class->save_thyself 	= GST_DEBUG_FUNCPTR (gst_bin_save_thyself);
@@ -133,6 +135,8 @@ gst_bin_class_init (GstBinClass * klass)
   gstelement_class->change_state 	= GST_DEBUG_FUNCPTR (gst_bin_change_state);
   gstelement_class->set_index	 	= GST_DEBUG_FUNCPTR (gst_bin_set_index);
 
+  klass->add_element 			= GST_DEBUG_FUNCPTR (gst_bin_add_func);
+  klass->remove_element 		= GST_DEBUG_FUNCPTR (gst_bin_remove_func);
   klass->iterate 			= GST_DEBUG_FUNCPTR (gst_bin_iterate_func);
 }
 
@@ -220,11 +224,10 @@ gst_bin_auto_clock (GstBin *bin)
 }
 
 static void
-gst_bin_set_index (GstBin *bin, GstIndex *index)
+gst_bin_set_index (GstElement *element, GstIndex *index)
 {
   GList *children;
-  
-  g_return_if_fail (GST_IS_BIN (bin));
+  GstBin *bin = GST_BIN (element);
 
   children = bin->children;
   while (children) {
@@ -267,9 +270,9 @@ gst_bin_set_element_sched (GstElement *element, GstScheduler *sched)
   /* otherwise, if it's just a regular old element */
   else {
     GList *pads;
-	  
+
     gst_scheduler_add_element (sched, element);
-	      
+
     /* set the sched pointer in all the pads */
     pads = element->pads;
     while (pads) {
@@ -277,7 +280,7 @@ gst_bin_set_element_sched (GstElement *element, GstScheduler *sched)
 
       pad = GST_PAD (pads->data);
       pads = g_list_next (pads);
-					                                  
+
       /* we only operate on real pads */
       if (!GST_IS_REAL_PAD (pad))
         continue;
@@ -285,12 +288,13 @@ gst_bin_set_element_sched (GstElement *element, GstScheduler *sched)
       /* if the peer element exists and is a candidate */
       if (GST_PAD_PEER (pad)) {
         if (gst_pad_get_scheduler (GST_PAD_PEER (pad)) == sched) {
-          GST_INFO (GST_CAT_SCHEDULING, "peer is in same scheduler, telling scheduler");
-		        
+          GST_INFO (GST_CAT_SCHEDULING,
+		    "peer is in same scheduler, telling scheduler");
+
           if (GST_PAD_IS_SRC (pad))
-            gst_scheduler_pad_connect (sched, pad, GST_PAD_PEER (pad));
+            gst_scheduler_pad_link (sched, pad, GST_PAD_PEER (pad));
           else
-            gst_scheduler_pad_connect (sched, GST_PAD_PEER (pad), pad);
+            gst_scheduler_pad_link (sched, GST_PAD_PEER (pad), pad);
         }
       }
     }
@@ -309,7 +313,7 @@ gst_bin_unset_element_sched (GstElement *element, GstScheduler *sched)
 	      GST_ELEMENT_NAME (element));
     return;
   }
-  
+
   GST_INFO (GST_CAT_SCHEDULING, "removing element \"%s\" from its sched %p",
 	    GST_ELEMENT_NAME (element), GST_ELEMENT_SCHED (element));
 
@@ -357,9 +361,9 @@ gst_bin_unset_element_sched (GstElement *element, GstScheduler *sched)
           GST_INFO (GST_CAT_SCHEDULING, "peer is in same scheduler, telling scheduler");
 
           if (GST_PAD_IS_SRC (pad))
-            gst_scheduler_pad_disconnect (sched, pad, GST_PAD_PEER (pad));
+            gst_scheduler_pad_unlink (sched, pad, GST_PAD_PEER (pad));
           else
-            gst_scheduler_pad_disconnect (sched, GST_PAD_PEER (pad), pad);
+            gst_scheduler_pad_unlink (sched, GST_PAD_PEER (pad), pad);
         }
       }
     }
@@ -397,35 +401,13 @@ gst_bin_add_many (GstBin *bin, GstElement *element_1, ...)
   va_end (args);
 }
 
-/**
- * gst_bin_add:
- * @bin: #GstBin to add element to
- * @element: #GstElement to add to bin
- *
- * Add the given element to the bin.  Set the elements parent, and thus
- * add a reference.
- */
-void
-gst_bin_add (GstBin *bin, GstElement *element)
+static void
+gst_bin_add_func (GstBin *bin, GstElement *element)
 {
   gint state_idx = 0;
   GstElementState state;
   GstScheduler *sched;
 
-  g_return_if_fail (bin != NULL);
-  g_return_if_fail (GST_IS_BIN (bin));
-  g_return_if_fail (element != NULL);
-  g_return_if_fail (GST_IS_ELEMENT (element));
-
-  GST_DEBUG (GST_CAT_PARENTAGE, "adding element \"%s\" to bin \"%s\"",
-	     GST_ELEMENT_NAME (element), GST_ELEMENT_NAME (bin));
-
-  /* must be not be in PLAYING state in order to modify bin */
-  g_return_if_fail (GST_STATE (bin) != GST_STATE_PLAYING);
-
-  /* the element must not already have a parent */
-  g_return_if_fail (GST_ELEMENT_PARENT (element) == NULL);
-
   /* then check to see if the element's name is already taken in the bin */
   if (gst_object_check_uniqueness (bin->children, 
 	                           GST_ELEMENT_NAME (element)) == FALSE)
@@ -460,33 +442,48 @@ gst_bin_add (GstBin *bin, GstElement *element)
 }
 
 /**
- * gst_bin_remove:
- * @bin: #GstBin to remove element from
- * @element: #GstElement to remove
+ * gst_bin_add:
+ * @bin: #GstBin to add element to
+ * @element: #GstElement to add to bin
  *
- * Remove the element from its associated bin, unparenting as well.
- * The element will also be unreferenced so there's no need to call
- * gst_object_unref on it.
- * If you want the element to still exist after removing, you need to call
- * #gst_object_ref before removing it from the bin.
+ * Add the given element to the bin.  Set the elements parent, and thus
+ * add a reference.
  */
 void
-gst_bin_remove (GstBin *bin, GstElement *element)
+gst_bin_add (GstBin *bin, GstElement *element)
 {
-  gint state_idx = 0;
-  GstElementState state;
-
-  GST_DEBUG_ELEMENT (GST_CAT_PARENTAGE, bin, "trying to remove child %s", GST_ELEMENT_NAME (element));
-
+  GstBinClass *bclass;
+  
   g_return_if_fail (bin != NULL);
   g_return_if_fail (GST_IS_BIN (bin));
   g_return_if_fail (element != NULL);
   g_return_if_fail (GST_IS_ELEMENT (element));
-  g_return_if_fail (bin->children != NULL);
 
-  /* must not be in PLAYING state in order to modify bin */
+  GST_DEBUG (GST_CAT_PARENTAGE, "adding element \"%s\" to bin \"%s\"",
+	     GST_ELEMENT_NAME (element), GST_ELEMENT_NAME (bin));
+
+  /* the element must not already have a parent */
+  g_return_if_fail (GST_ELEMENT_PARENT (element) == NULL);
+
+  /* must be not be in PLAYING state in order to modify bin */
   g_return_if_fail (GST_STATE (bin) != GST_STATE_PLAYING);
 
+  bclass = GST_BIN_GET_CLASS (bin);
+
+  if (bclass->add_element) {
+    bclass->add_element (bin, element);
+  }
+  else {
+    g_warning ("cannot add elements to bin %s\n", GST_ELEMENT_NAME (bin));
+  }
+}
+
+static void
+gst_bin_remove_func (GstBin *bin, GstElement *element)
+{
+  gint state_idx = 0;
+  GstElementState state;
+
   /* the element must have its parent set to the current bin */
   g_return_if_fail (GST_ELEMENT_PARENT (element) == (GstObject *) bin);
 
@@ -526,6 +523,43 @@ gst_bin_remove (GstBin *bin, GstElement *element)
   gst_object_unref (GST_OBJECT (element));
 }
 
+/**
+ * gst_bin_remove:
+ * @bin: #GstBin to remove element from
+ * @element: #GstElement to remove
+ *
+ * Remove the element from its associated bin, unparenting as well.
+ * The element will also be unreferenced so there's no need to call
+ * gst_object_unref on it.
+ * If you want the element to still exist after removing, you need to call
+ * #gst_object_ref before removing it from the bin.
+ */
+void
+gst_bin_remove (GstBin *bin, GstElement *element)
+{
+  GstBinClass *bclass;
+
+  GST_DEBUG_ELEMENT (GST_CAT_PARENTAGE, bin, "trying to remove child %s", GST_ELEMENT_NAME (element));
+
+  g_return_if_fail (bin != NULL);
+  g_return_if_fail (GST_IS_BIN (bin));
+  g_return_if_fail (element != NULL);
+  g_return_if_fail (GST_IS_ELEMENT (element));
+  g_return_if_fail (bin->children != NULL);
+
+  /* must not be in PLAYING state in order to modify bin */
+  g_return_if_fail (GST_STATE (bin) != GST_STATE_PLAYING);
+
+  bclass = GST_BIN_GET_CLASS (bin);
+
+  if (bclass->remove_element) {
+    bclass->remove_element (bin, element);
+  }
+  else {
+    g_warning ("cannot remove elements from bin %s\n", GST_ELEMENT_NAME (bin));
+  }
+}
+
 /**
  * gst_bin_child_state_change:
  * @bin: #GstBin with the child
@@ -893,6 +927,8 @@ gst_bin_iterate (GstBin *bin)
 
   oclass = GST_BIN_GET_CLASS (bin);
 
+  gst_object_ref (GST_OBJECT (bin));
+
   if (bin->pre_iterate_func)
     (bin->pre_iterate_func) (bin, bin->pre_iterate_data);
 
@@ -914,6 +950,7 @@ gst_bin_iterate (GstBin *bin)
       running = TRUE;
     }
   }
+  gst_object_unref (GST_OBJECT (bin));
 
   return running;
 }
